searchData={"content_type":"text/markdown","items":[{"doc":"Lightweight dependency injection using application config.\n\nDependable works by making defined behaviours callable by name. Dependable\nwill do the lookup of the implementation module for you and invoke it. This\nmakes it easier to see behaviour in application code as well as getting auto\ncomplete of the functions defined in the behaviour.\n\nAs an example, say we have the need for sending emails in our application. We\nmight define a behaviour such as:\n\n```elixir\ndefmodule MyApp.EmailProvider do\n  @callback send(MyApp.Email.t()) :: {:ok, MyApp.Email.t()} | {:error, MyApp.EmailSendError.t()}\nend\n```\n\nNow we can define an implementation for this module.\n\n```elixir\ndefmodule Infra.FancyEmailProvider do\n  @behaviour MyApp.EmailProvider\n\n  @impl MyApp.EmailProvider\n  def send(%MyApp.Email{} = email) do\n    # ...implementation details ...\n  end\nend\n```\n\nNext we can use application environment to configure what email provider is\nused. For this example we can imagine that in prod envs we would want the\n`FancyEmailProvider` but in testing environments we would want to inject a\nMox mock. For the key name in configuration we can use the name of the\nbehaviour itself which is the default lookup key used by Dependable.\n\n```elixir\n# in config.exs\nconfig :my_app, MyApp.EmailProvider, Infra.FancyEmailProvider\n\n# in test.exs\n# this mock would have to be defined in some tool such Mox\nconfig :my_app, MyApp.EmailProvider, MyApp.EmailProvider.Mock\n```\n\nLastly, we can now call into the behaviour in our application logic.\n\n```elixir\ndefmodule MyApp.Onboarding do\n  def onboard_customer(params) do\n    params\n      |> MyApp.Email.new!()\n      |> MyApp.EmailProvider.send()\n  end\nend\n```","ref":"Dependable.html","title":"Dependable","type":"module"},{"doc":"Lookup the implementor of a given behaviour raising if one cannot be found.","ref":"Dependable.html#impl!/2","title":"Dependable.impl!/2","type":"function"},{"doc":"Raised when an implementor of a behaviour cannot be found.\n\nEither the application config doesn't contain a entry for setting the\nimplmentor for a given env or the lookup key for the implementor is\nincorrect.","ref":"Dependable.ImplMissingError.html","title":"Dependable.ImplMissingError","type":"exception"},{"doc":"# Overview\n\nDependable is a lightweight library for dependency injection in Elixir using Application config. With the goal of making dependency injection\nand designing code with behaviours a better developer experience.\n\nIf you are unfamiliar with the concepts around dependency injection AppSignal has put together a great [blog post](https://blog.appsignal.com/2024/05/21/using-dependency-injection-in-elixir.html) for getting up to speed on the pattern and how it can be implemented in elixir.","ref":"overview.html","title":"Overview","type":"extras"},{"doc":"- Lightweight\n- Sensible defaults\n- Keep IDE features such as auto complete for behaviour callbacks","ref":"overview.html#benefits","title":"Benefits - Overview","type":"extras"},{"doc":"- Behaviours have to be defined in their own module. Due to how Dependable proxies functions calls the implementation of a behaviour and the behaviour itself cannot reside in the same module.\n\nThe remainder of this guide will be an getting started guide with getting dependable installed, configured, and then put to use.\n\n# Installation & Configuration\n\nDependable is available through hex.pm. You can look up the most up to date version by calling.\n\n```bash\nmix hex.search dependable\n```\n\nThen adding it to your application dependencies in `mix.exs`.\n\n```elixir\ndefp deps do\n  [\n    # ... snip\n    {:dependable, \"0.1.0\"}\n    # ... snip\n  ]\nend\n```\n\nNext, Dependable needs a bit of application configuration in order to lookup application config for your implementation modules. Place the name\nof your `:otp_app` (`:my_app` in this example) in your `config/config.exs`.\n\n```elixir\nconfig :dependable, :otp_app, :my_app\n```\n\nThis will be the default application config that will be queried when using Dependable,","ref":"overview.html#tradeoffs","title":"Tradeoffs - Overview","type":"extras"},{"doc":"Dependable works by making defined behaviours callable by name. Dependable will do the lookup of the implementation module for you and invoke it.\nThis makes it easier to see behaviour in application code as well as getting auto complete of the functions defined in the behaviour. As an\nexample, say we have the need for sending emails in our application. We might define a behaviour such as:\n\n```elixir\ndefmodule MyApp.EmailProvider do\n  use Dependable\n\n  @callback send(MyApp.Email.t()) :: {:ok, MyApp.Email.t()} | {:error, MyApp.EmailSendError.t()}\nend\n```\n\nNow we can define an implementation for this module.\n\n```elixir\ndefmodule Infra.FancyEmailProvider do\n  @behaviour MyApp.EmailProvider\n\n  @impl MyApp.EmailProvider\n  def send(%MyApp.Email{} = email) do\n    # ...implementation details ...\n  end\nend\n```\n\nNext we can use application environment to configure what email provider is used. For this example we can imagine that in prod envs we would want\nthe `FancyEmailProvider` but in testing environments we would want to inject a Mox mock. For the key name in configuration we can use the name\nof the behaviour itself which is the default lookup key used by Dependable.\n\n```elixir\n# in config/config.exs\nconfig :my_app, MyApp.EmailProvider, Infra.FancyEmailProvider\n\n# in config/test.exs\n# this mock would have to be defined in some tool such Mox\nconfig :my_app, MyApp.EmailProvider, MyApp.EmailProvider.Mock\n```\n\nLastly, we can now call into the behaviour in our application logic. Notice the function we invoke is on the behaviour module itself.\nDependable will do the work of looking up the underlying implementor and invoking it.\n\n```elixir\ndefmodule MyApp.Onboarding do\n  def onboard_customer(params) do\n    params\n      |> MyApp.Email.new!()\n      |> MyApp.EmailProvider.send()\n  end\nend\n```","ref":"overview.html#usage","title":"Usage - Overview","type":"extras"},{"doc":"By default Dependable will look for behaviour in the configured otp application namespace using the name of the behaviour as the lookup key. If\nfor whatever reason this default doesn't work it can be overriden on a case by case basis in the `use` statement. Such as:\n\n```elixir\ndefmodule MyApp.EmailProvider do\n  use Dependable, otp_app: :other_appliction, key: :email_provider\n\n  # snip...\nend\n```","ref":"overview.html#overriding-defaults","title":"Overriding defaults - Overview","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}